@@ -1,10 +1,17 @@
 /* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
 
+imports.gi.versions.Clutter = '1.0';
+imports.gi.versions.Gio = '2.0';
+imports.gi.versions.Gdk = '2.0';
+imports.gi.versions.GdkPixbuf = '2.0';
+imports.gi.versions.Gtk = '2.0';
+
 const Clutter = imports.gi.Clutter;
 const DBus = imports.dbus;
 const Gdk = imports.gi.Gdk;
 const Gio = imports.gi.Gio;
 const GLib = imports.gi.GLib;
+const GConf = imports.gi.GConf;
 const Lang = imports.lang;
 const Mainloop = imports.mainloop;
 const Meta = imports.gi.Meta;
@@ -23,37 +30,41 @@ const RunDialog = imports.ui.runDialog;
 const LookingGlass = imports.ui.lookingGlass;
 const NotificationDaemon = imports.ui.notificationDaemon;
 const WindowAttentionHandler = imports.ui.windowAttentionHandler;
+const Scripting = imports.ui.scripting;
 const ShellDBus = imports.ui.shellDBus;
-const Sidebar = imports.ui.sidebar;
+const TelepathyClient = imports.ui.telepathyClient;
 const WindowManager = imports.ui.windowManager;
+const Magnifier = imports.ui.magnifier;
 
 const DEFAULT_BACKGROUND_COLOR = new Clutter.Color();
 DEFAULT_BACKGROUND_COLOR.from_pixel(0x2266bbff);
 
 let chrome = null;
 let panel = null;
-let sidebar = null;
 let placesManager = null;
 let overview = null;
 let runDialog = null;
 let lookingGlass = null;
 let wm = null;
-let notificationDaemon = null;
 let messageTray = null;
+let notificationDaemon = null;
 let windowAttentionHandler = null;
+let telepathyClient = null;
 let recorder = null;
 let shellDBusService = null;
 let modalCount = 0;
 let modalActorFocusStack = [];
+let uiGroup = null;
+let magnifier = null;
 let _errorLogStack = [];
 let _startDate;
 
 let background = null;
 
 function start() {
-    // Add a binding for "global" in the global JS namespace; (gjs
+    // Add a binding for 'global' in the global JS namespace; (gjs
     // keeps the web browser convention of having that namespace be
-    // called "window".)
+    // called 'window'.)
     window.global = Shell.Global.get();
 
     // Now monkey patch utility functions into the global proxy;
@@ -62,7 +73,7 @@ function start() {
     global.logError = _logError;
     global.log = _logDebug;
 
-    Gio.DesktopAppInfo.set_desktop_env("GNOME");
+    Gio.DesktopAppInfo.set_desktop_env('GNOME');
 
     global.grab_dbus_service();
     shellDBusService = new ShellDBus.GnomeShell();
@@ -91,32 +102,40 @@ function start() {
     global.stage.color = DEFAULT_BACKGROUND_COLOR;
 
     let themeContext = St.ThemeContext.get_for_stage (global.stage);
-    let stylesheetPath = global.datadir + "/theme/gnome-shell.css";
+    let stylesheetPath = global.datadir + '/theme/gnome-shell.css';
     let theme = new St.Theme ({ application_stylesheet: stylesheetPath });
     themeContext.set_theme (theme);
 
     let shellwm = global.window_manager;
-    shellwm.takeover_keybinding("panel_main_menu");
-    shellwm.connect("keybinding::panel_main_menu", function () {
+    shellwm.takeover_keybinding('panel_main_menu');
+    shellwm.connect('keybinding::panel_main_menu', function () {
         overview.toggle();
     });
-    shellwm.takeover_keybinding("panel_run_dialog");
-    shellwm.connect("keybinding::panel_run_dialog", function () {
+    shellwm.takeover_keybinding('panel_run_dialog');
+    shellwm.connect('keybinding::panel_run_dialog', function () {
        getRunDialog().open();
     });
 
+    // Set up stage hierarchy to group all UI actors under one container.
+    uiGroup = new Clutter.Group();
+    global.window_group.reparent(uiGroup);
+    global.overlay_group.reparent(uiGroup);
+    global.stage.add_actor(uiGroup);
+
     placesManager = new PlaceDisplay.PlacesManager();
     overview = new Overview.Overview();
     chrome = new Chrome.Chrome();
     panel = new Panel.Panel();
-    sidebar = new Sidebar.Sidebar();
     wm = new WindowManager.WindowManager();
+    messageTray = new MessageTray.MessageTray();
     notificationDaemon = new NotificationDaemon.NotificationDaemon();
     windowAttentionHandler = new WindowAttentionHandler.WindowAttentionHandler();
-    messageTray = new MessageTray.MessageTray();
+    telepathyClient = new TelepathyClient.Client();
 
     _startDate = new Date();
 
+    let recorderSettings = new Gio.Settings({ schema: 'org.gnome.shell.recorder' });
+
     global.screen.connect('toggle-recording', function() {
         if (recorder == null) {
             recorder = new Shell.Recorder({ stage: global.stage });
@@ -125,11 +144,11 @@ function start() {
         if (recorder.is_recording()) {
             recorder.pause();
         } else {
-            //read the parameters from GConf always in case they have changed
-            let gconf = Shell.GConf.get_default();
-            recorder.set_framerate(gconf.get_int("recorder/framerate"));
-            recorder.set_filename("shell-%d%u-%c." + gconf.get_string("recorder/file_extension"));
-            let pipeline = gconf.get_string("recorder/pipeline");
+            // read the parameters from GSettings always in case they have changed
+            recorder.set_framerate(recorderSettings.get_int('framerate'));
+            recorder.set_filename('shell-%d%u-%c.' + recorderSettings.get_string('file-extension'));
+            let pipeline = recorderSettings.get_string('pipeline');
+
             if (!pipeline.match(/^\s*$/))
                 recorder.set_pipeline(pipeline);
             else
@@ -143,7 +162,7 @@ function start() {
     global.stage.add_actor(background);
     background.lower_bottom();
 
-    global.connect('screen-size-changed', _relayout);
+    global.gdk_screen.connect('monitors-changed', _relayout);
 
     ExtensionSystem.init();
     ExtensionSystem.loadExtensions();
@@ -155,10 +174,23 @@ function start() {
 
     global.stage.connect('captured-event', _globalKeyPressHandler);
 
+    // Install magnifier.
+    magnifier = new Magnifier.Magnifier();
+
+    // Perform initial relayout here
+    _relayout();
+
     _log('info', 'loaded at ' + _startDate);
     log('GNOME Shell started at ' + _startDate);
 
     Mainloop.idle_add(_removeUnusedWorkspaces);
+
+    let perfModuleName = GLib.getenv("SHELL_PERF_MODULE");
+    if (perfModuleName) {
+        let perfOutput = GLib.getenv("SHELL_PERF_OUTPUT");
+        let module = eval('imports.perf.' + perfModuleName + ';');
+        Scripting.runPerfScript(module, perfOutput);
+    }
 }
 
 /**
@@ -179,7 +211,7 @@ function _log(category, msg) {
         for (let i = 2; i < arguments.length; i++) {
             text += JSON.stringify(arguments[i]);
             if (i < arguments.length - 1)
-                text += " ";
+                text += ' ';
         }
     }
     _errorLogStack.push({timestamp: new Date().getTime(),
@@ -267,9 +299,9 @@ function _globalKeyPressHandler(actor, event) {
         let symbol = event.get_key_symbol();
         if (symbol == Clutter.Print) {
             // We want to be able to take screenshots of the shell at all times
-            let gconf = Shell.GConf.get_default();
-            let command = gconf.get_string("/apps/metacity/keybinding_commands/command_screenshot");
-            if (command != null && command != "") {
+            let gconf = GConf.Client.get_default();
+            let command = gconf.get_string('/apps/metacity/keybinding_commands/command_screenshot');
+            if (command != null && command != '') {
                 let [ok, len, args] = GLib.shell_parse_argv(command);
                 let p = new Shell.Process({'args' : args});
                 p.run();
@@ -342,7 +374,7 @@ function _findModal(actor) {
 function pushModal(actor) {
     if (modalCount == 0) {
         if (!global.begin_modal(global.get_current_time())) {
-            log("pushModal: invocation of begin_modal failed");
+            log('pushModal: invocation of begin_modal failed');
             return false;
         }
     }
@@ -485,7 +517,7 @@ function _queueBeforeRedraw(workId) {
         Meta.later_add(Meta.LaterType.BEFORE_REDRAW, function () {
             _runBeforeRedrawQueue();
             return false;
-        }, null);
+        });
     }
 }
 
@@ -509,7 +541,7 @@ function _queueBeforeRedraw(workId) {
  */
 function initializeDeferredWork(actor, callback, props) {
     // Turn into a string so we can use as an object property
-    let workId = "" + (++_deferredWorkSequence);
+    let workId = '' + (++_deferredWorkSequence);
     _deferredWorkData[workId] = { 'actor': actor,
                                   'callback': callback };
     actor.connect('notify::mapped', function () {
@@ -539,7 +571,7 @@ function initializeDeferredWork(actor, callback, props) {
 function queueDeferredWork(workId) {
     let data = _deferredWorkData[workId];
     if (!data) {
-        global.logError("invalid work id ", workId);
+        global.logError('invalid work id ', workId);
         return;
     }
     if (_deferredWorkQueue.indexOf(workId) < 0)
