@@ -25,8 +25,6 @@ typedef struct _ShellGlobalClass ShellGlobalClass;
 struct _ShellGlobalClass
 {
   GObjectClass parent_class;
-
-  void (*screen_size_changed) (ShellGlobal *global);
 };
 
 GType            shell_global_get_type            (void) G_GNUC_CONST;
@@ -40,6 +38,8 @@ ShellGlobal *shell_global_get (void);
 
 MetaScreen *shell_global_get_screen (ShellGlobal  *global);
 
+GdkScreen *shell_global_get_gdk_screen (ShellGlobal  *global);
+
 gboolean shell_global_add_extension_importer (ShellGlobal *global,
                                               const char  *target_object_script,
                                               const char  *target_property,
@@ -89,6 +89,8 @@ void shell_global_get_pointer (ShellGlobal         *global,
                                int                 *y,
                                ClutterModifierType *mods);
 
+GSettings *shell_global_get_settings (ShellGlobal *global);
+
 ClutterModifierType shell_get_event_state (ClutterEvent *event);
 
 void shell_popup_menu (GtkMenu *menu, int button, guint32 time,
